==> ideaIC-182.3208.16-sources.jar: com/intellij/debugger/engine/RemoteDebugProcessHandler.java <==
/*
 * Copyright 2000-2009 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.intellij.debugger.engine;

import com.intellij.debugger.DebuggerManager;
import com.intellij.execution.process.ProcessHandler;
import com.intellij.openapi.project.Project;

import java.io.OutputStream;

public class RemoteDebugProcessHandler extends ProcessHandler{
  private final Project myProject;

  public RemoteDebugProcessHandler(Project project) {
    myProject = project;
  }

  @Override
  public void startNotify() {
    final DebugProcess debugProcess = DebuggerManager.getInstance(myProject).getDebugProcess(this);
    final DebugProcessListener listener = new DebugProcessListener() {
      //executed in manager thread
      public void processDetached(DebugProcess process, boolean closedByUser) {
        debugProcess.removeDebugProcessListener(this);
        notifyProcessDetached();
      }
    };
    debugProcess.addDebugProcessListener(listener);
    try {
      super.startNotify();
    }
    finally {
      // in case we added our listener too late, we may have lost processDetached notification,
      // so check here if process is detached
      if (debugProcess.isDetached()) {
        debugProcess.removeDebugProcessListener(listener);
        notifyProcessDetached();
      }
    }
  }

  @Override
  protected void destroyProcessImpl() {
    DebugProcess debugProcess = DebuggerManager.getInstance(myProject).getDebugProcess(this);
    if(debugProcess != null) {
      debugProcess.stop(true);
    }
  }

  @Override
  protected void detachProcessImpl() {
    DebugProcess debugProcess = DebuggerManager.getInstance(myProject).getDebugProcess(this);
    if(debugProcess != null) {
      debugProcess.stop(false);
    }
  }

  @Override
  public boolean detachIsDefault() {
    return true;
  }

  @Override
  public OutputStream getProcessInput() {
    return null;
  }
}

==> ideaIC-2018.3.3-sources.jar: com/intellij/debugger/engine/RemoteDebugProcessHandler.java <==
// Copyright 2000-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.debugger.engine;

import com.intellij.debugger.DebuggerManager;
import com.intellij.execution.process.ProcessHandler;
import com.intellij.openapi.project.Project;

import java.io.OutputStream;
import java.util.concurrent.atomic.AtomicBoolean;

public class RemoteDebugProcessHandler extends ProcessHandler{
  private final Project myProject;
  private final boolean myAutoRestart;
  private final AtomicBoolean myClosedByUser = new AtomicBoolean();

  public RemoteDebugProcessHandler(Project project) {
    this(project, false);
  }

  public RemoteDebugProcessHandler(Project project, boolean autoRestart) {
    myProject = project;
    myAutoRestart = autoRestart;
  }

  @Override
  public void startNotify() {
    final DebugProcessListener listener = new DebugProcessAdapterImpl() {
      //executed in manager thread
      @Override
      public void processDetached(DebugProcessImpl process, boolean closedByUser) {
        if (!myAutoRestart || closedByUser || myClosedByUser.get()) {
          process.removeDebugProcessListener(this);
          notifyProcessDetached();
        }
        else {
          process.reattach(process.getSession().getDebugEnvironment());
        }
      }
    };
    DebugProcess debugProcess = DebuggerManager.getInstance(myProject).getDebugProcess(this);
    debugProcess.addDebugProcessListener(listener);
    try {
      super.startNotify();
    }
    finally {
      // in case we added our listener too late, we may have lost processDetached notification,
      // so check here if process is detached
      if (debugProcess.isDetached()) {
        debugProcess.removeDebugProcessListener(listener);
        notifyProcessDetached();
      }
    }
  }

  @Override
  protected void destroyProcessImpl() {
    myClosedByUser.set(true);
    DebugProcess debugProcess = DebuggerManager.getInstance(myProject).getDebugProcess(this);
    if(debugProcess != null) {
      debugProcess.stop(true);
    }
  }

  @Override
  protected void detachProcessImpl() {
    myClosedByUser.set(true);
    DebugProcess debugProcess = DebuggerManager.getInstance(myProject).getDebugProcess(this);
    if(debugProcess != null) {
      debugProcess.stop(false);
    }
  }

  @Override
  public boolean detachIsDefault() {
    return true;
  }

  @Override
  public OutputStream getProcessInput() {
    return null;
  }
}
